@@ -535,14 +535,20 @@ _init_stats_key_builders(AFProgramDestDriver *self, StatsClusterKeyBuilder **wri
   stats_cluster_key_builder_add_legacy_label(*writer_sck_builder, stats_cluster_label("command",
                                              self->process_info.cmdline->str));
 
-  *driver_sck_builder = stats_cluster_key_builder_clone(*writer_sck_builder);
+  *driver_sck_builder = stats_cluster_key_builder_new();
+  stats_cluster_key_builder_add_label(*driver_sck_builder, stats_cluster_label("driver", "program"));
   stats_cluster_key_builder_add_label(*driver_sck_builder, stats_cluster_label("id", self->super.super.id));
+  stats_cluster_key_builder_add_legacy_label(*driver_sck_builder, stats_cluster_label("command",
+                                             self->process_info.cmdline->str));
   stats_cluster_key_builder_set_legacy_alias(*driver_sck_builder,
                                              self->writer_options.stats_source | SCS_DESTINATION,
                                              self->super.super.id, self->process_info.cmdline->str);
 
-  *queue_sck_builder = stats_cluster_key_builder_clone(*writer_sck_builder);
+  *queue_sck_builder = stats_cluster_key_builder_new();
+  stats_cluster_key_builder_add_label(*queue_sck_builder, stats_cluster_label("driver", "program"));
   stats_cluster_key_builder_add_label(*queue_sck_builder, stats_cluster_label("id", self->super.super.id));
+  stats_cluster_key_builder_add_legacy_label(*queue_sck_builder, stats_cluster_label("command",
+                                             self->process_info.cmdline->str));
 }
 
 static gboolean
