@@ -31,7 +31,7 @@ extern CfgParser msg_generator_parser;
 extern CfgParser threaded_random_generator_parser;
 #endif
 
-#ifdef SYSLOG_NG_ENABLE_CPP
+#if SYSLOG_NG_ENABLE_CPP
 extern CfgParser random_choice_generator_parser;
 #endif
 
@@ -57,7 +57,7 @@ static Plugin example_plugins[] =
     .parser = &threaded_random_generator_parser,
   },
 #endif
-#ifdef SYSLOG_NG_ENABLE_CPP
+#if SYSLOG_NG_ENABLE_CPP
   {
     .type = LL_CONTEXT_SOURCE,
     .name = "random_choice_generator",
