@@ -29,6 +29,8 @@
 #include "apphook.h"
 #include "python-dest.h"
 #include "python-main.h"
+#include "python-startup.h"
+#include "python-global.h"
 #include "python-fetcher.h"
 #include "python-source.h"
 #include "python-bookmark.h"
@@ -117,9 +119,9 @@ Test(python_persist_name, test_python_dest)
   _load_code(python_destination_code);
 
   LogDriver *d = python_dd_new(empty_cfg);
-  python_dd_set_class(d, "Dest");
+  python_binding_set_class(python_dd_get_binding(d), "Dest");
 
-  _add_dummy_option(python_dd_get_options(d));
+  _add_dummy_option(python_dd_get_binding(d)->options);
 
   cr_assert(log_pipe_init((LogPipe *)d));
 
@@ -144,9 +146,9 @@ Test(python_persist_name, test_python_fetcher)
   _load_code(python_fetcher_code);
 
   LogDriver *d = python_fetcher_new(empty_cfg);
-  python_fetcher_set_class(d, "Fetcher");
+  python_binding_set_class(python_fetcher_get_binding(d), "Fetcher");
 
-  _add_dummy_option(python_fetcher_get_options(d));
+  _add_dummy_option(python_fetcher_get_binding(d)->options);
 
   cr_assert(log_pipe_init((LogPipe *)d));
 
@@ -173,9 +175,9 @@ Test(python_persist_name, test_python_source)
   _load_code(python_source_code);
 
   LogDriver *d = python_sd_new(empty_cfg);
-  python_sd_set_class(d, "Source");
+  python_binding_set_class(python_sd_get_binding(d), "Source");
 
-  _add_dummy_option(python_sd_get_options(d));
+  _add_dummy_option(python_sd_get_binding(d)->options);
 
   cr_assert(log_pipe_init((LogPipe *)d));
 
@@ -246,9 +248,10 @@ Test(python_persist_name, test_python_fetcher_no_generate_persist_name)
   _load_code(python_fetcher_code_no_generate_persist_name);
 
   LogDriver *d = python_fetcher_new(empty_cfg);
-  python_fetcher_set_class(d, "Fetcher");
 
-  _add_dummy_option(python_fetcher_get_options(d));
+  python_binding_set_class(python_fetcher_get_binding(d), "Fetcher");
+
+  _add_dummy_option(python_fetcher_get_binding(d)->options);
 
   log_pipe_set_persist_name((LogPipe *)d, "test_persist_name");
   cr_assert(log_pipe_init((LogPipe *)d));
@@ -273,9 +276,9 @@ Test(python_persist_name, test_python_source_no_generate_persist_name)
   _load_code(python_source_code_no_generate_persist_name);
 
   LogDriver *d = python_sd_new(empty_cfg);
-  python_sd_set_class(d, "Source");
+  python_binding_set_class(python_sd_get_binding(d), "Source");
 
-  _add_dummy_option(python_sd_get_options(d));
+  _add_dummy_option(python_sd_get_binding(d)->options);
 
   log_pipe_set_persist_name((LogPipe *)d, "test_persist_name");
   cr_assert(log_pipe_init((LogPipe *)d));
@@ -306,7 +309,7 @@ Test(python_persist_name, test_python_source_readonly)
   _load_code(python_source_code_readonly);
 
   LogDriver *d = python_sd_new(empty_cfg);
-  python_sd_set_class(d, "Source");
+  python_binding_set_class(python_sd_get_binding(d), "Source");
   start_grabbing_messages();
   cr_assert_eq(log_pipe_init((LogPipe *)d), 0);
   stop_grabbing_messages();
@@ -337,7 +340,7 @@ Test(python_persist_name, test_python_fetcher_readonly)
   _load_code(python_fetcher_code_readonly);
 
   LogDriver *d = python_fetcher_new(empty_cfg);
-  python_fetcher_set_class(d, "Fetcher");
+  python_binding_set_class(python_fetcher_get_binding(d), "Fetcher");
 
   start_grabbing_messages();
   cr_assert_eq(log_pipe_init((LogPipe *)d), 0);
@@ -359,9 +362,9 @@ Test(python_persist_name, test_python_fetcher_persist_preference)
 
   LogDriver *d = python_fetcher_new(empty_cfg);
   log_pipe_set_persist_name(&d->super, "test_persist_name");
-  python_fetcher_set_class(d, "Fetcher");
+  python_binding_set_class(python_fetcher_get_binding(d), "Fetcher");
 
-  _add_dummy_option(python_fetcher_get_options(d));
+  _add_dummy_option(python_fetcher_get_binding(d)->options);
 
   cr_assert(log_pipe_init((LogPipe *)d));
 
@@ -379,9 +382,9 @@ Test(python_persist_name, test_python_source_persist_preference)
 
   LogDriver *d = python_sd_new(empty_cfg);
   log_pipe_set_persist_name(&d->super, "test_persist_name");
-  python_sd_set_class(d, "Source");
+  python_binding_set_class(python_sd_get_binding(d), "Source");
 
-  _add_dummy_option(python_sd_get_options(d));
+  _add_dummy_option(python_sd_get_binding(d)->options);
 
   cr_assert(log_pipe_init((LogPipe *)d));
 
